/*
 * Copyright 2019 Banco Bilbao Vizcaya Argentaria, S.A.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package mux

import (
	"net/http"

	"github.com/gorilla/mux"

	"github.com/BBVA/kapow/internal/logger"
	"github.com/BBVA/kapow/internal/server/model"
)

func gorillize(rs []model.Route, buildHandler func(model.Route) http.Handler) *mux.Router {
	m := mux.NewRouter()

	for _, r := range rs {
		m.Handle(r.Pattern, buildHandler(r)).Methods(r.Method)
	}
	m.NotFoundHandler = http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		w.WriteHeader(http.StatusNotFound)
		logger.LogAccess(
			r.RemoteAddr,
			"-",
			"-",
			r.Method,
			r.RequestURI,
			r.Proto,
			http.StatusNotFound,
			0,
			r.Header.Get("Referer"),
			r.Header.Get("User-Agent"),
		)
	})
	m.MethodNotAllowedHandler = http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		w.WriteHeader(http.StatusMethodNotAllowed)
		logger.LogAccess(
			r.RemoteAddr,
			"-",
			"-",
			r.Method,
			r.RequestURI,
			r.Proto,
			http.StatusMethodNotAllowed,
			0,
			r.Header.Get("Referer"),
			r.Header.Get("User-Agent"),
		)
	})
	return m
}
